var LineChart = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

LineChart.prototype = {
	init : function() {
		initReferences(this);
		var me = this;
		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI : function(obj) {
		var params = this._params;
		var width = obj.width();
		var height = obj.height();
		var opaque = params.bgo_0 > 0;
		
		if (params.shadow) {
			obj.css("overflow", "visible");	
		}

		// create canvas
		var canvas = createCanvas(width, height, opaque ? undefined : params);
		obj.empty().append(canvas);
		var ctx = opaque ? canvas.getContext('2d') : getCanvasContext(canvas, params);
		
		// draw background
		ctx.beginPath();
		ctx.rect(0, 0, width, height);
		fillAndStrokeOnCanvas(ctx, width, height, params, false, true);
		
		// draw border
		ctx.beginPath();
		var borderColor = params.bdc_0;
		var borderWidthTop = params.bd_t_0;
		var borderWidthRight = params.bd_r_0;
		var borderWidthBottom = params.bd_b_0;
		var borderWidthLeft = params.bd_l_0;
		var swingInterval = params.bdsw_int_0;
		var swingAmplitude = params.bdsw_amp_0;
		var offset = Math.floor(swingAmplitude / 2);
		ctx.strokeStyle = borderColor;
		drawSwingBorder(ctx, offset, offset, width - 2 * offset, height - 2 * offset, borderWidthTop, borderWidthRight, borderWidthBottom, borderWidthLeft, 0, 0, 0, 0, swingInterval, swingAmplitude);
		
		// draw lines
		var points = params.points;
		var colors = params.colors;
		var margins = params.margins;
		var bounds = params.bounds;
		var chartWidth = bounds.right - bounds.left + 1;
		var chartHeight = bounds.bottom - bounds.top + 1;
		var ratioX = (width - margins.left - margins.right) / chartWidth;
		var ratioY = (height - margins.top - margins.bottom) / chartHeight;
		for (var i = 0; i < colors.length; i ++) {
			ctx.beginPath();
			ctx.strokeStyle = colors[i];
			var linePoints = points[i];
			for (var j = 1; j < linePoints.length; j ++) {
				var prevPt = linePoints[j - 1];
				var curPt = linePoints[j];
				var x1 = margins.left + (prevPt.x - bounds.left) * ratioX;
				var y1 = height - 1 - margins.bottom - (prevPt.y - bounds.top) * ratioY;
				var x2 = margins.left + (curPt.x - bounds.left) * ratioX;
				var y2 = height - 1 - margins.bottom - (curPt.y - bounds.top) * ratioY;
				drawSwingLine(ctx, { x : x1, y : y1 }, { x : x2, y : y2 }, swingInterval, swingAmplitude, j > 1, true);
			}
		}
		
		// shadow
		if (opaque) {
			applyShadow(obj, $(canvas), params.shadow);
		}
	}
};