var ProgressBar = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

ProgressBar.prototype = {
	init: function() {
		var params = this._params;
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"></div>")
			.appendTo(this._obj);
		this.setupBehavior(this._obj);
		initReferences(this);
		var me = this;
		setTimeout(function() {
			updateElementAndReferences(me);
		}, 0);
	},
	updateUI: function(obj) {
		var params = this._params;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var height = obj.height();
		
		// background filling
		_bd.css("border-width", "0px").width(width).height(height);
		generateBackgroundAndBorders(0, "", _bg, _bd, params);
		
		// indicator
		var tile = params.tile;
		var indicatorColor = params.indc;
		var indicatorImage = params.indi;
		var indicatorWidth = params.indw;
		var indicatorMargin = params.indm;
		var v = width * params.cur / (params.max - params.min); 
		if (tile) {
			var indicator = $("<div></div>").css({
				"position" : "absolute",
				"left" : indicatorMargin + "px",
				"top" : indicatorMargin + "px",
				"width" : (v - indicatorMargin * 2) + "px",
				"height" : (height - indicatorMargin * 2) + "px",
				"overflow" : "hidden",
				"white-space" : "nowrap"
			}).appendTo(_fg.empty());
			if (indicatorImage != null) {
				var indImg = new Image();
				indImg.onload = function() {
					var imgWidth = this.width;
					var imgHeight = height - indicatorMargin * 2;
					for (var i = indicatorMargin; i < width; i += imgWidth) {
						$("<img src=\"" + indImg.src + "\" width=\"" + imgWidth + "\" height=\"" + imgHeight + "\">").appendTo(indicator);
					}
				}
				indImg.src = "images/" + indicatorImage;
			} else if (indicatorColor != null) {
				if (indicatorColor.indexOf("#") == 0) {
					indicator.css("background-color", indicatorColor);
				} else {
					indicator.empty().append($("<img src=\"images/" + indicatorColor + "\" width=\"" + indicator.width() + "\" height=\"100%\" />"));
				}
			}
		} else {
			if (indicatorWidth > 0) {
				var startX = Math.max(v - indicatorWidth / 2, indicatorMargin);
				var endX = Math.min(v + indicatorWidth / 2, width - 1 - indicatorMargin);
				var indicator = $("<div></div>").css({
					"position" : "absolute",
					"left" : startX + "px",
					"top" : indicatorMargin + "px",
					"width" : (endX - startX + 1) + "px",
					"height" : (height - indicatorMargin * 2) + "px",
					"overflow" : "hidden",
					"white-space" : "nowrap"
				}).appendTo(_fg.empty());
				if (indicatorImage != null) {
					var indImg = new Image();
					indImg.onload = function() {
						var imgWidth = this.width;
						var imgHeight = height - indicatorMargin * 2;
						var left = indicatorWidth > width ? (imgWidth * v / width - imgWidth) : 0;
						for (var i = startX; i < endX + imgWidth; i += imgWidth) {
							$("<img src=\"" + indImg.src + "\" width=\"" + imgWidth + "\" height=\"" + imgHeight + "\">")
								.css({
									"position" : "relative",
									"left" : left + "px"
								})
								.appendTo(indicator);
						}
					}
					indImg.src = "images/" + indicatorImage;
				} else if (indicatorColor != null) {
					if (indicatorColor.indexOf("#") == 0) {
						indicator.css("background-color", indicatorColor);
					} else {
						indicator.empty().append($("<img src=\"images/" + indicatorColor + "\" width=\"" + indicatorWidth + "\" height=\"100%\" />"));
					}
				}
			}
		}
		
		// shadow
		applyShadow(this._obj, _bd, params.shadow);
	},
	setupBehavior : function(obj) {
		clearInterval(this.interval);
		var me = this;
		var params = this._params;
		if (params.indt) {
			this.interval = setInterval(function() {
				var newVal = params.cur + params.step;
				if (newVal > params.max) {
					newVal = params.min;
				}
				me.setCurrentValue(newVal);
			}, 300);
		}
	},
	setCurrentValue : function(value) {
		var params = this._params;
		if (value > params.max) {
			value = params.max;
		} else if (value < params.min) {
			value = params.min;
		}
		if (params.cur != value) {
			params.cur = value;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Value_Changed");
		}
	},
	getCurrentValue : function() {
		return this._params.cur;
	},
	getMaxValue : function() {
		return this._params.max;
	}
};