var ScrollBar = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

ScrollBar.prototype = {
	init: function() {
		var params = this._params;
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"><div class=\"db\"></div><div class=\"ib\"></div><div class=\"track\"></div><div class=\"thumb\"></div></div>")
			.appendTo(this._obj);
		this.setupBehavior(this._obj);
		initReferences(this);
		var me = this;
		setTimeout(function() {
			setState(me, me._params.state, true);
		}, 0);
	},
	updateUI: function(obj) {
		var params = this._params;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var height = obj.height();
		var state = params.state;
		var vertical = params.vert;
		var buttonPlacement = params["btnp_" + state];
		var trackMargin = params["trackm_" + state];
		
		// background filling
		_bd.css("border-width", "0px").width(width).height(height);
		generateBackgroundAndBorders(state, "", _bg, _bd, params);
		
		// calculate targets' bounds
		var max = Math.max(width, height);
		var min = Math.min(width, height);
		var decreaseButtonRc = {};
		var increaseButtonRc = {};
		var trackRc = {};
		var thumbRc = {};
		if (buttonPlacement == "both") {
			decreaseButtonRc.x = 0;
			decreaseButtonRc.y = 0;
			decreaseButtonRc.width = decreaseButtonRc.height = min;
			increaseButtonRc.x = max - min;
			increaseButtonRc.y = 0;
			increaseButtonRc.width = increaseButtonRc.height = min;
			trackRc.x = decreaseButtonRc.x + decreaseButtonRc.width + trackMargin;
		} else if (buttonPlacement == "left") {
			decreaseButtonRc.x = 0;
			decreaseButtonRc.y = 0;
			decreaseButtonRc.width = decreaseButtonRc.height = min;
			increaseButtonRc.x = min;
			increaseButtonRc.y = 0;
			increaseButtonRc.width = increaseButtonRc.height = min;
			trackRc.x = increaseButtonRc.x + increaseButtonRc.width + trackMargin;
		} else {
			decreaseButtonRc.x = max - min * 2;
			decreaseButtonRc.y = 0;
			decreaseButtonRc.width = decreaseButtonRc.height = min;
			increaseButtonRc.x = max - min;
			increaseButtonRc.y = 0;
			increaseButtonRc.width = increaseButtonRc.height = min;
			trackRc.x = trackMargin;
		}
		trackRc.y = 0;
		trackRc.width = max - (min + trackMargin) * 2;
		trackRc.height = min;
		thumbRc.x = trackRc.x + trackRc.width * params.cur / params.max;
		thumbRc.y = 0;
		thumbRc.width = trackRc.width * params.view / params.max;
		thumbRc.height = min;
		if (vertical) {
			rotateRectangle(decreaseButtonRc);
			rotateRectangle(increaseButtonRc);
			rotateRectangle(trackRc);
			rotateRectangle(thumbRc);
		}
		
		// decrease button
		var db = _fg.children(".db").css("position", "absolute");
		var dbbdt = params["dbbd_t_" + state];
		var dbbdl = params["dbbd_l_" + state];
		var dbbdb = params["dbbd_b_" + state];
		var dbbdr = params["dbbd_r_" + state];
		db.css({
			"left" : decreaseButtonRc.x + "px",
			"top" : decreaseButtonRc.y + "px",
			"width" : decreaseButtonRc.width + "px",
			"height" : decreaseButtonRc.height + "px",
			"border-width" : "0px",
			"cursor" : "pointer"
		});
		generateBackgroundAndBorders(state, "db", db, db, params);
		var dbi = params["dbi_" + state];
		if (dbi) {
			var dbIcon = new Image();
			dbIcon.onload = function() {
				var x = (decreaseButtonRc.width - dbbdl - dbbdr - this.width) / 2;
				var y = (decreaseButtonRc.height - dbbdt - dbbdb - this.height) / 2;
				db.children(".icon").remove();
				db.append($("<img class=\"icon\" style=\"position:absolute; left:" + x + "px; top:" + y + "px;\" src=\"" + dbIcon.src + "\" />"));
			};
			dbIcon.src = "images/" + dbi;
		}
		
		// increase button
		var ib = _fg.children(".ib").css("position", "absolute");
		var ibbdt = params["ibbd_t_" + state];
		var ibbdl = params["ibbd_l_" + state];
		var ibbdb = params["ibbd_b_" + state];
		var ibbdr = params["ibbd_r_" + state];
		ib.css({
			"left" : increaseButtonRc.x + "px",
			"top" : increaseButtonRc.y + "px",
			"width" : increaseButtonRc.width + "px",
			"height" : increaseButtonRc.height + "px",
			"border-width" : "0px",
			"cursor" : "pointer"
		});
		generateBackgroundAndBorders(state, "ib", ib, ib, params);
		var ibi = params["ibi_" + state];
		if (ibi) {
			var ibIcon = new Image();
			ibIcon.onload = function() {
				var x = (increaseButtonRc.width - ibbdl - ibbdr - this.width) / 2;
				var y = (increaseButtonRc.height - ibbdt - ibbdb - this.height) / 2;
				ib.children(".icon").remove();
				ib.append($("<img class=\"icon\" style=\"position:absolute; left:" + x + "px; top:" + y + "px;\" src=\"" + ibIcon.src + "\" />"));
			};
			ibIcon.src = "images/" + ibi;
		}
		
		// track
		var track = _fg.children(".track").css("position", "absolute");
		var trackbdt = params["trackbd_t_" + state];
		var trackbdl = params["trackbd_l_" + state];
		var trackbdb = params["trackbd_b_" + state];
		var trackbdr = params["trackbd_r_" + state];
		track.css({
			"left" : trackRc.x + "px",
			"top" : trackRc.y + "px",
			"width" : trackRc.width + "px",
			"height" : trackRc.height + "px",
			"border-width" : "0px"
		});
		generateBackgroundAndBorders(state, "track", track, track, params);
		var tracki = params["tracki_" + state];
		if (tracki) {
			var trackIcon = new Image();
			trackIcon.onload = function() {
				var x = (trackRc.width - trackbdl - trackbdr - this.width) / 2;
				var y = (trackRc.height - trackbdt - trackbdb - this.height) / 2;
				track.children(".icon").remove();
				track.append($("<img class=\"icon\" style=\"position:absolute; left:" + x + "px; top:" + y + "px;\" src=\"" + trackIcon.src + "\" />"));
			};
			trackIcon.src = "images/" + tracki;
		}
		
		// thumb
		var thumb = _fg.children(".thumb").css("position", "absolute");
		var thumbbdt = params["thumbbd_t_" + state];
		var thumbbdl = params["thumbbd_l_" + state];
		var thumbbdb = params["thumbbd_b_" + state];
		var thumbbdr = params["thumbbd_r_" + state];
		thumb.css({
			"left" : thumbRc.x + "px",
			"top" : thumbRc.y + "px",
			"width" : thumbRc.width + "px",
			"height" : thumbRc.height + "px",
			"border-width" : "0px",
			"cursor" : "pointer"
		});
		generateBackgroundAndBorders(state, "thumb", thumb, thumb, params);
		var thumbi = params["thumbi_" + state];
		if (thumbi) {
			var thumbIcon = new Image();
			thumbIcon.onload = function() {
				var x = (thumbRc.width - thumbbdl - thumbbdr - this.width) / 2;
				var y = (thumbRc.height - thumbbdt - thumbbdb - this.height) / 2;
				thumb.children(".icon").remove();
				thumb.append($("<img class=\"icon\" style=\"position:absolute; left:" + x + "px; top:" + y + "px;\" src=\"" + thumbIcon.src + "\" />"));
			};
			thumbIcon.src = "images/" + thumbi;
		}
		
		// enable & disable
		_fg.children(".thumb").draggable((state & 64) == 0 ? "enable" : "disable");
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
	},
	setupBehavior : function(obj) {
		var me = this;
		var params = this._params;
		var fg = obj.children(".fg");
		var track = fg.children(".track")
		var thumb = fg.children(".thumb");
		var db = fg.children(".db");
		var ib = fg.children(".ib");
		var dragging = false;
		thumb.draggable({
			containment : track,
			cursor : "pointer",
			scroll : false,
			axis : params.vert ? "y" : "x",
			start : function() {
				dragging = true;
			},
			stop : function(){
				setTimeout(function() {
					var newVal;
					if (params.vert) {
						newVal = params.max * (thumb.position().top - track.position().top) / track.height();
					} else {
						newVal = params.max * (thumb.position().left - track.position().left) / track.width();
					}
					me.setCurrentValue(Math.round(newVal));
					dragging = false;
					setState(me, 0);
				}, 50);
			}
		});
		thumb.mouseenter(function() {
			if (!dragging) {
				setState(me, 16);
			}
		}).mouseleave(function() {
			if (!dragging) {
				setState(me, 0);
			}
		}).mousedown(function(e) {
			e.stopPropagation();
		});
		db.click(function() {
			me.setCurrentValue(params.cur - params.step);
		});
		ib.click(function() {
			me.setCurrentValue(params.cur + params.step);
		});
	},
	setCurrentValue : function(value) {
		var params = this._params;
		if (value > params.max - params.view) {
			value = params.max - params.view;
		} else if (value < params.min) {
			value = params.min;
		}
		if (params.cur != value) {
			params.cur = value;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Value_Changed");
		}
	},
	getCurrentValue : function() {
		return this._params.cur;
	},
	getMaxValue : function() {
		return this._params.max;
	},
	getViewSize : function() {
		return this._params.view;
	}
};