var Slider = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Slider.prototype = {
	init: function() {
		var params = this._params;
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"><img /></div>")
			.appendTo(this._obj);
		this.setupBehavior(this._obj);
		initReferences(this);
		var me = this;
		setTimeout(function() {
			setState(me, me._params.state, true);
		}, 0);
	},
	updateUI: function(obj) {
		var params = this._params;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var height = obj.height();
		var state = params.state;
		var vertical = params.vert;
		var trackSize = params.track;
		var trackX = (width - trackSize) / 2;
		var trackY = (height - trackSize) / 2;
		
		// background filling
		var trackRc = params.bounds;
		var trackCss = {
			"left" : trackRc.x + "px",
			"top" : trackRc.y + "px",
			"width" : (vertical ? trackRc.width : width) + "px",
			"height" : (vertical ? height : trackRc.height) + "px",
			"border-width" : "0px"
		};
		_bg.css(trackCss);
		_bd.css(trackCss);
		generateBackgroundAndBorders(state, "", _bg, _bd, params);
		
		// thumb
		var thumbImage = params[(vertical ? "vert" : "hor") + "_timg_" + state];
		if (thumbImage) {
			var thickness = params.size;
			var thumbLock = params["tlock_" + state];
			var ratio = params.cur / params.max;
			var thumbImg = new Image();
			thumbImg.onload = function() {
				var imgWidth = vertical ? thickness : this.width * (thumbLock ? thickness / this.height : 1);
				var imgHeight = vertical ? this.height * (thumbLock ? thickness / this.width : 1) : thickness;
				var imgX = vertical ? (thickness - imgWidth) / 2 : (width - imgWidth) * ratio;
				var imgY = vertical ? (height - imgHeight) * ratio : (thickness - imgHeight) / 2;
				_fg.children("img").css({
					"cursor" : "pointer",
					"left" : imgX + "px",
					"top" : imgY + "px"
				}).attr({
					"src" : thumbImg.src,
					"width" : imgWidth,
					"height" : imgHeight
				});
			};
			thumbImg.src = "images/" + thumbImage;
		}
		
		// enable & disable
		var thumbObj = _fg.children("img").draggable((state & 64) == 0 ? "enable" : "disable");
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
		applyDropShadow(obj, thumbObj, params);
	},
	setupBehavior : function(obj) {
		var me = this;
		var params = this._params;
		var fg = obj.children(".fg");
		var thumb = fg.children("img");
		var dragging = false;
		thumb.draggable({
			containment: "parent",
			axis: params.vert ? "y" : "x",
			cursor : "pointer",
			scroll : false ,
			start : function() {
				dragging = true;
			},
			stop : function(event, ui) {
				setTimeout(function() {
					var newVal;
					if (params.vert) {
						newVal = params.max * thumb.position().top / (fg.height() - thumb.height());
					} else {
						newVal = params.max * thumb.position().left / (fg.width() - thumb.width());
					}
					me.setCurrentValue(Math.round(newVal));
					dragging = false;
				}, 50);
			}
		});
		thumb.mouseenter(function() {
			if (!dragging) {
				setState(me, 16);
			}
		}).mouseleave(function() {
			if (!dragging) {
				setState(me, 0);
			}
		}).mousedown(function(e) {
			e.stopPropagation();
		});
	},
	setCurrentValue : function(value) {
		var params = this._params;
		if (value > params.max) {
			value = params.max;
		} else if (value < params.min) {
			value = params.min;
		}
		if (params.cur != value) {
			params.cur = value;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Value_Changed");
		}
	},
	getCurrentValue : function() {
		return this._params.cur;
	},
	getMaxValue : function() {
		return this._params.max;
	}
};