var Spinner = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Spinner.prototype = {
	init : function() {
		var params = this._params;
		this._text = this._obj.html();
		this._obj.empty();
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"></div>")
			.appendTo(this._obj);
		
		var value = params.values[params.selIndex];
		var margin = params.margin;
		$("<input class=\"edit\" type=\"text\" value=\"" + value + "\" />")
			.css({
				"width" : "100%",
				"height" : "100%",
				"padding-left" : margin,
				"padding-right" : margin,
				"background-color" : "transparent",
				"border-width" : "0px",
				"overflow" : "hidden"
			})
			.appendTo(this._fg);
		$("<div class=\"upButton\"><img style=\"position:absolute;width:100%;height:100%\" /></div>").appendTo(this._fg);
		$("<div class=\"downButton\"><img style=\"position:absolute;width:100%;height:100%\" /></div>").appendTo(this._fg);

		this.setupBehavior(this._obj);
		initReferences(this);

		var me = this;
		setTimeout(function() {
			setState(me, me._params.state, true);
		}, 0);
	},
	updateUI : function(obj) {
		var params = this._params;
		var state = params.state;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var height = obj.height();
		var buttonWidth = height * params.whRatio;
		var editorWidth = params.btnInBounds ? width : width - buttonWidth;
		
		// background filling
		_bd.css("border-width", "0px").width(editorWidth).height(height);
		generateBackgroundAndBorders(state, "", _bg, _bd, params);
		
		// text
		var margin = params.margin;
		var value = params.values[params.selIndex];
		_fg.find(".edit").val(value).width(editorWidth - margin * 2).height(height);
		applyFontParameters(state, "", _fg.find(".edit"), params);
		
		// buttons
		var btnMargin = params.btnMargin;
		var btnWidth = buttonWidth - btnMargin * 2;
		var btnHeight = height / 2 - btnMargin;
		_fg.children(".upButton").css({
			"position" : "absolute",
			"left" : (width - buttonWidth + btnMargin) + "px",
			"top" : btnMargin + "px",
			"width" : btnWidth + "px",
			"height" : btnHeight + "px"
		}).children("img").attr("src", "images/" + params["upBtnImg_" + state]);
		_fg.children(".downButton").css({
			"position" : "absolute",
			"left" : (width - buttonWidth + btnMargin) + "px",
			"top" : (height / 2) + "px",
			"width" : btnWidth + "px",
			"height" : btnHeight + "px"
		}).children("img").attr("src", "images/" + params["downBtnImg_" + state]);
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
	},
	setupBehavior : function(obj) {
		var me = this;
		var params = this._params;
		var count = params.values.length;
		var _fg = obj.children(".fg");
		_fg.children(".upButton").click(function() {
			if ((me._params.state & 64) == 0 && params.selIndex > 0) {
				params.selIndex --;
				updateElementAndReferences(me);
				$("#" + me._id).trigger("Value_Changed");
			}
		});
		_fg.children(".downButton").click(function() {
			if ((me._params.state & 64) == 0 && params.selIndex < count - 1) {
				params.selIndex ++;
				updateElementAndReferences(me);
				$("#" + me._id).trigger("Value_Changed");
			}
		});
		_fg.children(".edit").focus(function() {
			window.foreui.focusedElementId = me._id;
			onPropertyValueChanged("FOCUSED_ELEMENT_ID", me._id);
		});
	},
	getValue : function() {
		var params = this._params;
		return params.values[params.selIndex];
	},
	setFocus : function() {
		var edit = this._obj.children(".fg").find(".edit").get(0);
		edit.focus();
		edit.select();
	},
	setCurrentValue : function(value) {
		var params = this._params;
		var oldVal = params.values[params.selIndex];
		if (oldVal != value) {
			var pos = $.inArray(value, params.values);
			if (pos != -1) {
				params.selIndex = pos;
			} else {
				params.values.splice(params.selIndex, 0, value);
			}
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Value_Changed");
		}
	}
};