var Table = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Table.prototype = {
	init: function() {
		var params = this._params;
		var content = this._obj.children();
		this._obj.empty();
		this._bg = $("<div class=\"bg\"></div>")
			.appendTo(this._obj);
		this._bd = $("<div class=\"bd\"></div>")
			.appendTo(this._obj);
		this._fg = $("<div class=\"fg\"></div>")
			.appendTo(this._obj);
			
		var viewCss = {
			"position" : "absolute",
			"overflow" : "hidden"
		};
		var tableCss = {
			"position" : "absolute",
			"table-layout" : "fixed",
			"border-collapse" : "collapse"
		};
		var tableAttr = {
			"border" : "0",
			"cellspacing" : "0"
		}
		
		// create left top corner header
		var leftTopHeader = $("<table class=\"ltHeader\"></table>")
			.css(tableCss)
			.attr(tableAttr);
		if (params.chdr && params.rhdr) {
			var firstRow = content.children().children("tr").attr("valign", "top").first();
			var firstCell = firstRow.children("td").first().remove();
			firstCell.css("padding", "0px");
			var labelBg = $("<div class=\"labelBg\" style=\"position:absolute;\"></div>");
			var label = $("<table class=\"label\" style=\"position:absolute; top:0px;\"><tr><td>" + firstCell.html() + "</td></tr></table>");
			firstCell.empty().append(labelBg).append(label);
			$("<tr valign=\"top\"></tr>").append(firstCell).appendTo(leftTopHeader);
		}
		
		// create column header
		var columnHeaderView = $("<div class=\"columnHeaderView\"></div>").css(viewCss).appendTo(this._fg);
		var columnHeader = $("<table class=\"columnHeader\"></table>")
			.css(tableCss)
			.attr(tableAttr)
			.appendTo(columnHeaderView);
		if (params.chdr) {
			var firstRow = content.children().children("tr").attr("valign", "top").first().remove();
			firstRow.children("td").each(function(col) {
				var hdr = $(this).css("padding", "0px").css("margin", "0px");
				var labelBg = $("<div class=\"labelBg\" style=\"position:absolute;\"></div>");
				var label = $("<table class=\"label\" style=\"position:absolute; top:0px;\"><tr><td class=\"table_cell\">" + hdr.html() + "</td></tr></table>");
				hdr.empty().append(labelBg).append(label);
			});
			firstRow.appendTo(columnHeader);
			var colCount = params.cols - (params.rhdr ? 1 : 0);
			for (var i = 0; i < colCount; i ++) {
				columnHeader.prepend($("<col />"));
			}
		}
		
		// append left top corner header
		leftTopHeader.appendTo(this._fg);
		
		// create right top corner header
		var rightTopHeader = $("<div class=\"rtHeader\"></div>").css(viewCss).appendTo(this._fg);
		
		// create row header
		var rowHeaderView = $("<div class=\"rowHeaderView\"></div>").css(viewCss);
		var rowHeader = $("<table class=\"rowHeader\"></table>")
			.css(tableCss)
			.attr(tableAttr)
			.appendTo(rowHeaderView);
		if (params.rhdr) {
			content.children().children("tr").each(function(index) {
				var hdr = $(this).children("td").first().remove();
				rowHeader.append("<tr valign=\"top\"><td><div class=\"labelBg\" style=\"position:absolute;\"></div><table class=\"label\" style=\"position:absolute; left:0px;\"><tr><td class=\"table_cell\">" + hdr.html() + "</td></tr></table></td></tr>");
			});
			rowHeader.prepend($("<col />"));
		}
		
		// create content
		var contentView = $("<div class=\"contentView\"></div>").css(viewCss).appendTo(this._fg);
		contentView.css({
			"overflow-x" : params.horsc ? "scroll" : "hidden",
			"overflow-y" : params.versc ? "scroll" : "hidden"
		});
		content.css(tableCss)
			.addClass("content")
			.attr(tableAttr)
			.appendTo(contentView);
		var startCol = params.rhdr ? 1 : 0;
		for (var i = startCol; i < params.cols; i ++) {
			content.prepend($("<col />"));
		}
		content.children().children("tr").each(function(row) {
			var row = $(this).css("padding", "0px").attr("valign", "top");
			row.children("td").each(function(col) {
				var cell = $(this).css("padding", "0px");
				var labelBg = $("<div class=\"labelBg\" style=\"position:absolute;\"></div>");
				var label = $("<table class=\"label\" style=\"position:absolute;display:block;\"><tr><td class=\"table_cell\">" + cell.html() + "</td></tr></table>");
				cell.empty().append(labelBg).append(label);
			});
		});
		
		// append column header
		rowHeaderView.appendTo(this._fg);
		
		// create left bottom corner header
		var leftBottomHeader = $("<div class=\"lbHeader\"></div>").css(viewCss).appendTo(this._fg);
		
		this.setupBehavior(this._obj);
		initReferences(this);

		var me = this;
		setTimeout(function() {
			setState(me, me._params.state, true);
			setTimeout(function() {
				setState(me, me._params.state, true);
			}, 0);
		}, 0);
	},
	updateUI: function(obj) {
		var me = this;
		var params = this._params;
		var state = params.state;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var borderType = params["bdt_" + state];
		var borderWidth = params["bdw_" + state];
		var borderReduce = borderType == "swing" ? "0" : borderWidth;
		var swingAmplitude = params["bdsw_amp_" + state];
		var scrollBarSize = getScrollBarSize();
		var scrollBarWidth = params.versc ? scrollBarSize : 0;
		var scrollBarHeight = params.horsc ? scrollBarSize : 0;
		var width = obj.width() - scrollBarWidth - borderWidth * 2 - swingAmplitude;
		var height = obj.height() - scrollBarHeight - borderWidth * 2 - swingAmplitude;
		var startRow = params.chdr ? 1 : 0;
		var startCol = params.rhdr ? 1 : 0;
		var selRow = params.selr;
		var selCol = params.selc;
		var rowCount = params.rows;
		var colCount = params.cols;
		
		var defaultRowHeight = params.rh;
		var rowHeights = params.rowh;
		var paddingHeight = height;
		for (var i = 0; i < rowHeights.length; i ++) {
			paddingHeight -= rowHeights[i];
		}
		paddingHeight = Math.max(0, paddingHeight);
		var paddingRowCount = Math.ceil(paddingHeight / defaultRowHeight);
		var lastPaddingRowHeight = paddingHeight - defaultRowHeight * (paddingRowCount - 1);
		var colWidths = [];
		var totalColumnsWidth = 0;
		for (var i = 0; i < params.colw.length; i ++) {
			colWidths[i] = Math.round(width * params.colw[i] / params.orgw);
			totalColumnsWidth += colWidths[i];
		}
		var ltHeaderWidth = (params.chdr && params.rhdr) ? colWidths[0] : 0;
		var ltHeaderHeight = (params.chdr && params.rhdr) ? rowHeights[0] : 0;
		var columnHeaderHeight = params.chdr ? rowHeights[0] : 0;
		var rowHeaderWidth = params.rhdr ? colWidths[0] : 0;
		var textMargin = params.txtm;
		var paintBorder = params["pbdr_" + state];
		var paintHorLines = params["horl_" + state];
		var paintVerLines = params["verl_" + state];
		var updatingSelection = params._updatingSelection;
		
		// background and border
		_bd.css("border-width", "0px").width(obj.width()).height(obj.height());
		if (!paintBorder) {
			params["bdt_" + state] = "none";
		}
		var mainBg = params["bg"];
		var mainBgState = params["bg_" + state];
		generateBackgroundAndBorders(state, "", _bg, _bd, params);
		
		if (!updatingSelection) {
		// left top corner header
		var leftTopHeader = _fg.children(".ltHeader");
		leftTopHeader.css({
			"left" : (borderWidth + swingAmplitude / 2) + "px",
			"top" : (borderWidth + swingAmplitude / 2) + "px",
			"width" : Math.ceil(ltHeaderWidth - borderWidth + swingAmplitude) + "px",
			"height" : (ltHeaderHeight - borderWidth - swingAmplitude) + "px"
		});
		leftTopHeader.children().children("tr").each(function(row) {
			var hdrHeight = ltHeaderHeight - borderReduce * 2 - swingAmplitude; 
			$(this).height(hdrHeight)
				.css("padding", "0px")
				.children("td").each(function(col) {
					var hdr = $(this);
					var bdt = params["chbd_t_" + state];
					var bdl = params["chbd_l_" + state];
					var bdb = params["chbd_b_" + state];
					var bdr = params["chbd_r_" + state];
					params["chbd_t_" + state] = params["chbd_l_" + state] = 0;
					if (!paintHorLines) {
						params["chbd_b_" + state] = 0;
					}
					if (!paintVerLines) {
						params["chbd_r_" + state] = 0;
					}
					var hdrWidth = Math.ceil(ltHeaderWidth - borderWidth * 2);
					var labelBg = hdr.children(".labelBg").width(hdrWidth).height(hdrHeight);
					generateBackgroundAndBorders(state, "ch", labelBg, hdr, params);
					params["chbd_t_" + state] = bdt;
					params["chbd_l_" + state] = bdl;
					params["chbd_b_" + state] = bdb;
					params["chbd_r_" + state] = bdr;
					hdr.children(".linearBg").css({
							"position" : "absolute", 
							"width" : (hdrWidth + (paintVerLines ? 0 : borderWidth)) + "px"
						});
					var hdrLabel = hdr.children(".label").width(hdrWidth).height(hdrHeight);
					var hdrLabelTd = hdrLabel.children().children("tr").children("td");
					hdrLabelTd.css({
						"padding-left" : textMargin + "px",
						"padding-right" : textMargin + "px"
					});
					me._formatCell(0, 0, state, "ch", hdrLabelTd, params);
				});
		});
		
		// right top corner header
		var rightTopHeader = _fg.children(".rtHeader");
		rightTopHeader.css({
			"left" : (obj.width() - 1 - scrollBarWidth - borderWidth - swingAmplitude * 2) + "px",
			"top" : (borderWidth + swingAmplitude / 2) + "px",
			"width" : (scrollBarWidth + borderWidth) + "px",
			"height" : colHdrHeight + "px",
			"visibility" : (params.chdr && params.versc) ? "inherit" : "hidden"
		});
		generateBackgroundAndBorders(state, "ch", rightTopHeader, rightTopHeader, params);
		
		// column header
		var colHdrHeight = columnHeaderHeight - borderReduce - swingAmplitude;
		var columnHeaderView = _fg.children(".columnHeaderView");
		columnHeaderView.css({
			"left" : Math.max(borderWidth, ltHeaderWidth) + "px",
			"top" : (borderWidth + swingAmplitude / 2) + "px",
			"width" : (width - ltHeaderWidth) + "px",
			"height" : colHdrHeight + "px"
		});
		var columnHeader = columnHeaderView.children(".columnHeader").width(totalColumnsWidth - ltHeaderWidth).height(colHdrHeight);
		columnHeader.children("col").each(function(index) {
			$(this).attr("width", colWidths[index + (params.rhdr ? 1 : 0)] + "");
		});
		columnHeader.children().children("tr").each(function(row) {
			var rowElem = $(this);
			rowElem.height(colHdrHeight - borderReduce)
				.css("padding", "0px")
				.children("td").add(rightTopHeader).each(function(col) {
					var hasPrevCell = (col > 0);
					var hasNextCell = (col < colCount - 1 - startCol);
					var hdr = $(this).height(colHdrHeight).width(colWidths[col + (params.rhdr ? 1 : 0)]);
					var bdt = params["chbd_t_" + state];
					var bdl = params["chbd_l_" + state];
					var bdb = params["chbd_b_" + state];
					var bdr = params["chbd_r_" + state];
					params["chbd_t_" + state] = 0;
					if (!paintHorLines) {
						params["chbd_b_" + state] = 0;
					}
					if (!paintVerLines || !hasPrevCell || params["chbdt_" + state] == "swing") {
						params["chbd_l_" + state] = 0;
					}
					if (!paintVerLines || !hasNextCell) {
						params["chbd_r_" + state] = 0;
					}
					var hdrWidth = hdr.hasClass("rtHeader") ? scrollBarWidth : hdr.width() + ($.browser.webkit && !hasPrevCell ? borderWidth : 0);
					if (rowElem.hasClass("paddingRow")) {
						generateBackgroundAndBorders(state, "ch", hdr, hdr, params);
					} else {
						var labelBg = hdr.children(".labelBg").width(hdrWidth).height(colHdrHeight - borderWidth);
						generateBackgroundAndBorders(state, "ch", labelBg, hdr, params);
					}
					params["chbd_t_" + state] = bdt;
					params["chbd_l_" + state] = bdl;
					params["chbd_b_" + state] = bdb;
					params["chbd_r_" + state] = bdr;
					hdr.children(".linearBg").css({
							"position" : "absolute", 
							"width" : (hdrWidth + (paintVerLines ? 0 : borderWidth)) + "px",
							"height" : (colHdrHeight - borderReduce + (paintHorLines ? 0 : borderWidth)) + "px"
						});
					var hdrLabel = hdr.children(".label").width(hdrWidth).height(colHdrHeight - borderWidth);
					var hdrLabelTd = hdrLabel.children().children("tr").children("td")
					hdrLabelTd.css({
						"padding-left" : textMargin + "px",
						"padding-right" : textMargin + "px"
					});
					me._formatCell(0, col + startCol, state, "ch", hdrLabelTd, params);
				});
		});
		
		// left bottom corner header
		var leftBottomHeader = _fg.children(".lbHeader");
		leftBottomHeader.css({
			"left" : (borderWidth + swingAmplitude / 2) + "px",
			"top" : height + "px",
			"width" : (rowHeaderWidth - borderWidth + swingAmplitude) + "px",
			"height" : (scrollBarHeight + borderWidth) + "px",
			"visibility" : (params.horsc && params.rhdr) ? "inherit" : "hidden"
		});
		generateBackgroundAndBorders(state, "rh", leftBottomHeader, leftBottomHeader, params);
		
		// row header
		var rowHeaderView = _fg.children(".rowHeaderView");
		rowHeaderView.css({
			"left" : (borderWidth + swingAmplitude / 2) + "px",
			"top" : (columnHeaderHeight + 1 - startRow) + "px",
			"width" : (rowHeaderWidth - borderWidth + swingAmplitude) + "px",
			"height" : (height - columnHeaderHeight + borderWidth - 1 + startRow) + "px"
		});
		var rowHeader = rowHeaderView.children(".rowHeader");
		rowHeader.children("col").width(colWidths[0]);
		rowHeader.children().children(".paddingRow").remove();
		for (var i = 0; i < paddingRowCount; i ++) {
			var paddingRow = $("<tr class=\"paddingRow\" valign=\"top\"></tr>").height(defaultRowHeight).css("padding", "0px");
			paddingRow.append($("<td></td>"));
			rowHeader.append(paddingRow);
		}
		rowHeader.children().children("tr").each(function(row) {
			var rowElem = $(this);
			var isFirstRow = (row == 0);
			var isLastRow = (row == rowHeights.length + paddingRowCount - startRow - 1);
			var rowHeight = rowHeights[startRow + row];
			if (!rowHeight) {
				rowHeight = (row == rowHeights.length + paddingRowCount - startRow - 1) ? lastPaddingRowHeight : defaultRowHeight;
			} else if (isLastRow) {
				rowHeight += borderWidth;
			}
			rowElem.height(rowHeight)
				.css("padding", "0px")
				.children("td").add(leftBottomHeader).each(function(col) {
					var hdr = $(this).css("padding", "0px");
					var bdt = params["rhbd_t_" + state];
					var bdl = params["rhbd_l_" + state];
					var bdb = params["rhbd_b_" + state];
					var bdr = params["rhbd_r_" + state];
					params["rhbd_l_" + state] = 0;
					if (!paintVerLines) {
						params["rhbd_r_" + state] = 0;
					}
					if ((isFirstRow && leftBottomHeader != this) || !paintHorLines) {
						params["rhbd_t_" + state] = 0;
					}
					if (isLastRow || !paintHorLines) {
						params["rhbd_b_" + state] = 0;
					}
					if (params["rhbdt_" + state] == "swing" && leftBottomHeader != this) {
						params["rhbd_t_" + state] = 0;
					}
					var hdrWidth = colWidths[0] - borderWidth * 2 + (hdr.hasClass("lbHeader") ? swingAmplitude : 0);
					var hdrHeight = hdr.height();
					if (rowElem.hasClass("paddingRow")) {
						generateBackgroundAndBorders(state, "rh", hdr, hdr, params);
					} else {
						var labelBg = hdr.children(".labelBg").width(hdrWidth).height(hdrHeight);
						generateBackgroundAndBorders(state, "rh", labelBg, hdr, params);
					}
					params["rhbd_t_" + state] = bdt;
					params["rhbd_l_" + state] = bdl;
					params["rhbd_b_" + state] = bdb;
					params["rhbd_r_" + state] = bdr;
					hdr.width(hdrWidth);
					hdr.children(".linearBg").css({
							"position" : "absolute",
							"width" : (hdrWidth + (paintVerLines ? 0 : borderWidth)) + "px"
						});
					var hdrLabel = hdr.children(".label").width(hdrWidth).height(hdrHeight);
					var hdrLabelTd = hdrLabel.children().children("tr").children("td");
					hdrLabelTd.css({
						"padding-left" : textMargin + "px",
						"padding-right" : textMargin + "px"
					});
					me._formatCell(row + startRow, 0, state, "rh", hdrLabelTd, params);
				});
		});
		}
		
		// content
		var contentView = _fg.children(".contentView");
		contentView.css({
			"left" : Math.max(borderWidth, rowHeaderWidth) + "px",
			"top" : (columnHeaderHeight + (params.chdr ? 0 : borderWidth * 2)) + "px",
			"width" : (width - rowHeaderWidth + scrollBarWidth + (params.rhdr ? borderWidth : 0)) + "px",
			"height" : (height - columnHeaderHeight + borderWidth + scrollBarHeight - (params.chdr ? 0 : borderWidth)) + "px"
		});
		var content = contentView.children(".content").width(totalColumnsWidth - (params.rhdr ? colWidths[0] : 0));
		content.children("col").each(function(index) {
			$(this).attr("width", colWidths[startCol + index] + "");
		});
		if (!updatingSelection) {
			content.children().children(".paddingRow").remove();
			for (var i = 0; i < paddingRowCount; i ++) {
				var paddingRow = $("<tr class=\"paddingRow\" valign=\"top\"></tr>").height(defaultRowHeight).css("padding", "0px");
				for (var j = 0; j < colWidths.length - startCol; j ++) {
					paddingRow.append($("<td></td>"));
				}
				content.append(paddingRow);
			}
		}
		content.children().children("tr").each(function(row) {
			var rowElem = $(this);
			var isFirstRow = (row == 0);
			var isLastRow = (row == rowHeights.length + paddingRowCount - startRow - 1);
			var rowHeight = rowHeights[startRow + row];
			if (!rowHeight) {
				rowHeight = isLastRow ? lastPaddingRowHeight : defaultRowHeight;
			} else {
				if (isFirstRow && !params.chdr) {
					rowHeight -= borderWidth;
				}
				if (isLastRow) {
					rowHeight -= borderWidth * 2;
				}
			}
			rowElem.height(rowHeight)
				.css("padding", "0px")
				.children("td").each(function(col) {
					if (updatingSelection && row + startRow != params._oldSelRow && row + startRow != selRow && col + startCol != params._oldSelCol && col + startCol != selCol) {
						return;
					}
					var hasPrevCell = (col > 0);
					var hasNextCell = (col < colCount - 1 - startCol);
					var cell = $(this).css("padding", "0px").width(colWidths[col + (params.rhdr ? 1 : 0)]);
					var tarId = "cell";
					if ((row + startRow) % 2 != startRow) {
						tarId = "alt";
					}
					if ((params.selm == 0 && row + startRow == selRow)
						|| (params.selm == 1 && col + startCol == selCol)
						|| (params.selm == 2 && row + startRow == selRow && col + startCol == selCol)) {
						tarId = "sel";
					}
					var bdt = params[tarId + "bd_t_" + state];
					var bdl = params[tarId + "bd_l_" + state];
					var bdb = params[tarId + "bd_b_" + state];
					var bdr = params[tarId + "bd_r_" + state];
					if (isFirstRow || !paintHorLines) {
						params[tarId + "bd_t_" + state] = 0;
					}
					if (isLastRow || !paintHorLines) {
						params[tarId + "bd_b_" + state] = 0;
					}
					if (!hasPrevCell || !paintVerLines) {
						params[tarId + "bd_l_" + state] = 0;
					}
					if (!hasNextCell || !paintVerLines) {
						params[tarId + "bd_r_" + state] = 0;
					}
					if (params[tarId + "bdt_" + state] == "swing") {
						params[tarId + "bd_l_" + state] = params[tarId + "bd_t_" + state] = 0;
					}
					var cellBg = params[tarId + "bg"];
					var cellBgState = params[tarId + "bg_" + state];
					if (tarId == "cell") {
						params[tarId + "bg_" + state] = mainBg || mainBgState;
					}
					var cellWidth = colWidths[col + startCol];
					var cellHeight = rowHeight - borderWidth;
					if (rowElem.hasClass("paddingRow")) {
						generateBackgroundAndBorders(state, tarId, cell, cell, params);
					} else {
						var cellW = cell.width() + ($.browser.webkit && !hasPrevCell ? borderWidth : 0)
						var labelBg = cell.children(".labelBg").width(cellW).height(isLastRow && startRow == 0 ? cellHeight : cell.height());
						generateBackgroundAndBorders(state, tarId, labelBg, cell, params);
					}
					params[tarId + "bg"] = cellBg;
					params[tarId + "bg_" + state] = cellBgState;
					params[tarId + "bd_t_" + state] = bdt;
					params[tarId + "bd_l_" + state] = bdl;
					params[tarId + "bd_b_" + state] = bdb;
					params[tarId + "bd_r_" + state] = bdr;
					cell.children(".linearBg").css({
							"position" : "absolute",
							"width" : (cellWidth + (paintVerLines ? 0 : borderWidth)) + "px",
							"height" : cellHeight + "px"
						});
					var cellRow = cell.children(".label").width(cellWidth).height(cellHeight).children().children("tr");
					var cellLabel = cellRow.children("td").width(cellWidth).height(cellHeight);
					cellLabel.css({
						"padding-left" : textMargin + "px",
						"padding-right" : textMargin + "px"
					});
					me._formatCell(row + startRow, col + startCol, state, tarId, cellLabel, params);
				});
		});
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
	},
	_formatCell : function(row, col, state, tarId, obj, params) {
		var align = params.align;
		var valign = params.valign;
		var format = params.cfmt["r" + row + "c" + col];
		if (format != undefined) {
			if (format.align != undefined) {
				align = format.align == -1 ? "left" : (format.align == 0 ? "center" : "right");
			}
			if (format.valign != undefined) {
				valign = format.valign == -1 ? "top" : (format.valign == 0 ? "middle" : "bottom");
			}
		}
		obj.attr({
			"align" : align,
			"valign" : valign
		});
		var bkFontName = params[tarId + "f_n_" + state];
		var bkFontSize = params[tarId + "f_s_" + state];
		var bkTextColor = params[tarId + "f_c_" + state];
		var bkBold = params[tarId + "f_b_" + state];
		var bkItalic = params[tarId + "f_i_" + state];
		var bkUnderline = params[tarId + "f_u_" + state];
		if (format != undefined) {
			if (format.fontName != undefined) {
				params[tarId + "f_n_" + state] = format.fontName;
			}
			if (format.fontSize != undefined) {
				params[tarId + "f_s_" + state] = format.fontSize;
			}
			if (format.color != undefined) {
				params[tarId + "f_c_" + state] = format.color;
			}
			if (format.bold != undefined) {
				params[tarId + "f_b_" + state] = format.bold;
			}
			if (format.italic != undefined) {
				params[tarId + "f_i_" + state] = format.italic;
			}
			if (format.underline != undefined) {
				params[tarId + "f_u_" + state] = format.underline;
			}
		}
		applyFontParameters(state, tarId, obj, params);
		params[tarId + "f_n_" + state] = bkFontName;
		params[tarId + "f_s_" + state] = bkFontSize;
		params[tarId + "f_c_" + state] = bkTextColor;
		params[tarId + "f_b_" + state] = bkBold;
		params[tarId + "f_i_" + state] = bkItalic;
		params[tarId + "f_u_" + state] = bkUnderline;
	},
	setupBehavior : function(obj) {
		var me = this;
		var params = this._params;
		var startRow = params.chdr ? 1 : 0;
		var startCol = params.rhdr ? 1 : 0;
		var _fg = obj.children(".fg");
		var columnHeaderView = _fg.children(".columnHeaderView");
		var rowHeaderView = _fg.children(".rowHeaderView");
		var contentView = _fg.children(".contentView");
		contentView.scroll(function() {
			columnHeaderView.scrollLeft(contentView.scrollLeft());
			rowHeaderView.scrollTop(contentView.scrollTop());
		});
		var content = contentView.children(".content");
		content.children().children("tr").each(function(rowIndex) {
			var rowElem = $(this);
			$(this).children("td").each(function(col) {
				$(this).click(function() {
					var bkSelRow = params.selr;
					var bkSelCol = params.selc;
					var row = rowElem.index();
					params.selr = params.selm != 1 ? row + startRow : -1;
					params.selc = params.selm != 0 ? col + startCol : -1;
					if (bkSelRow != params.selr || bkSelCol != params.selc) {
						params._updatingSelection = true;
						params._oldSelRow = bkSelRow;
						params._oldSelCol = bkSelCol;
						updateElementAndReferences(me);
						params._updatingSelection = false;
						$("#" + me._id).trigger("Selection_Changed");
					}
				});
			});
		});
	},
	_setupBehaviorForRow : function(rowElem) {
		var me = this;
		var params = this._params;
		var startRow = params.chdr ? 1 : 0;
		var startCol = params.rhdr ? 1 : 0;
		rowElem.children("td").each(function(col) {
			$(this).click(function() {
				var bkSelRow = params.selr;
				var bkSelCol = params.selc;
				var row = rowElem.index();
				params.selr = params.selm != 1 ? row + startRow : -1;
				params.selc = params.selm != 0 ? col + startCol : -1;
				if (bkSelRow != params.selr || bkSelCol != params.selc) {
					params._updatingSelection = true;
					params._oldSelRow = bkSelRow;
					params._oldSelCol = bkSelCol;
					updateElementAndReferences(me);
					params._updatingSelection = false;
					$("#" + me._id).trigger("Selection_Changed");
				}
			});
		});
	},
	getSelectedIndex : function() {
		var params = this._params;
		return (params.selr == -1) ? 0 : params.selr + 1 - (params.chdr ? 1 : 0);
	},
	getSelectedColumn : function() {
		var params = this._params;
		return (params.selc == -1) ? 0 : params.selc + 1 - (params.rhdr ? 1 : 0);
	},
	getCellValues : function() {
		var values = [undefined];
		this._obj.children(".fg").children(".contentView").children(".content").children().children("tr").each(function(row) {
			var rowElem = $(this);
			if (rowElem.hasClass("paddingRow")) {
				return false;
			}
			var rowValues = [undefined];
			rowElem.children("td").each(function(col) {
				rowValues.push(textFromView($(this).find(".table_cell").text()));
			});
			values.push(rowValues);
		});
		return values;
	},
	getRowCount : function() {
		return this._obj.children(".fg").children(".contentView").children(".content").children().children("tr").size();
	},
	getActualRowCount : function() {
		return this._obj.children(".fg").children(".contentView").children(".content").children().children("tr:not(.paddingRow)").size();
	},
	getColumnCount : function() {
		return this._obj.children(".fg").children(".contentView").children(".content").children("col").size();
	},
	setSelectedIndex : function(index) {
		var tarIndex = index - 1 + (this._params.rhdr ? 1 : 0);
		if (this._params.selr != tarIndex) {
			this._params.selr = tarIndex;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Selection_Changed");
		}
	},
	setSelectedColumn : function(targets, index) {
		for (var i = 0; i < targets.length; i ++) {
			var elem = getForeUIElementById(targets[i]);
			if (elem) {
				var tarIndex = index - 1 + (elem._params.chdr ? 1 : 0);
				if (elem._params.selc != tarIndex) {
					elem._params.selc = tarIndex;
					updateElementAndReferences(elem);
					$("#" + elem._id).trigger("Selection_Changed");
				}
			}
		}
	},
	setCellValue : function(targets, row, col, value) {
		for (var i = 0; i < targets.length; i ++) {
			var elem = getForeUIElementById(targets[i]);
			if (elem && elem.getRowCount() >= row && elem.getColumnCount() >= col) {
				$("[ref=\"" + elem._id + "\"]").add(elem._obj).children(".fg").each(function() {
					var tableRow = $(this).children(".contentView").children(".content").children().children("tr").get(row - 1);
					var cell = $(tableRow).children("td").get(col - 1);
					$(cell).find(".table_cell").html(htmlToView(value));
				});
			}
		}
	},
	appendRow : function(targets, data) {
		data = "" + data;
		for (var i = 0; i < targets.length; i ++) {
			var elem = getForeUIElementById(targets[i]);
			if (elem) {
				if (elem._updateTask) {
					clearTimeout(elem._updateTask);
					elem._updateCounter = 0;
				}
				var params = elem._params;
				var cells = data.split(",");
				var newHdr = null;
				if (params.rhdr) {
					newHdr = $("<tr valign=\"top\"><td><div class=\"labelBg\" style=\"position:absolute;\"></div><table class=\"label\" style=\"position:absolute; left:0px;\"><tr><td class=\"table_cell\">" + cells[0] + "</td></tr></table></td></tr>");
					cells.splice(0, 1);
				}
				params.rowh.push(params.rh);
				var newRow = $("<tr valign=\"top\"></tr>");
				for (i = 0; i < cells.length; i ++) {
					var labelBg = $("<div class=\"labelBg\" style=\"position:absolute;\"></div>");
					var label = $("<table class=\"label\" style=\"position:absolute;display:block;\"><tr><td class=\"table_cell\">" + cells[i] + "</td></tr></table>");
					newRow.append($("<td></td>").append(labelBg).append(label));
				}
				$("[ref=\"" + elem._id + "\"]").add(elem._obj).children(".fg").each(function() {
					var _fg = $(this);
					if (newHdr != null) {
						var rowHeader = _fg.children(".rowHeaderView").children(".rowHeader").children("tbody");
						rowHeader.append(newHdr.clone());
					}
					var contentBody = _fg.children(".contentView").children(".content").children("tbody");
					var newRowCopy = newRow.clone();
					contentBody.append(newRowCopy);
					elem._setupBehaviorForRow(newRowCopy);
				});
				elem._updateCounter = 2;
				elem._updateTask = setTimeout(function() {
					updateElementAndReferences(elem);
					elem._updateCounter --;
					if (elem._updateCounter > 0) {
						elem._updateTask = setTimeout(function() {
							updateElementAndReferences(elem);
							elem._updateCounter --;
						}, 0);
					} else {
						elem._updateTask = false;
					}
				}, 250);
			}
		}
	},
	insertRow : function(targets, row, data) {
		data = "" + data;
		for (var i = 0; i < targets.length; i ++) {
			var elem = getForeUIElementById(targets[i]);
			if (elem) {
				var rowCount = elem.getRowCount();
				if (rowCount == 0 || rowCount < row) {
					elem.appendRow(targets, data);
				} else {
					if (elem._updateTask) {
						clearTimeout(elem._updateTask);
						elem._updateCounter = 0;
					}
					var params = elem._params;
					var cells = data.split(",");
					var newHdr = null;
					if (params.rhdr) {
						newHdr = $("<tr valign=\"top\"><td><div class=\"labelBg\" style=\"position:absolute;\"></div><table class=\"label\" style=\"position:absolute; left:0px;\"><tr><td class=\"table_cell\">" + cells[0] + "</td></tr></table></td></tr>");
						cells.splice(0, 1);
					}
					params.rowh.push(params.rh);
					var newRow = $("<tr valign=\"top\"></tr>");
					for (i = 0; i < cells.length; i ++) {
						var labelBg = $("<div class=\"labelBg\" style=\"position:absolute;\"></div>");
						var label = $("<table class=\"label\" style=\"position:absolute;display:block;\"><tr><td class=\"table_cell\">" + cells[i] + "</td></tr></table>");
						newRow.append($("<td></td>").append(labelBg).append(label));
					}
					$("[ref=\"" + elem._id + "\"]").add(elem._obj).children(".fg").each(function() {
						var _fg = $(this);
						if (newHdr != null) {
							var rowHeader = _fg.children(".rowHeaderView").children(".rowHeader").children("tbody");
							newHdr.clone().insertBefore(rowHeader.children().eq(row - 1));
						}
						var contentBody = _fg.children(".contentView").children(".content").children("tbody");
						var newRowCopy = newRow.clone();
						newRowCopy.insertBefore(contentBody.children().eq(row - 1));
						elem._setupBehaviorForRow(newRowCopy);
					});
					elem._updateCounter = 2;
					elem._updateTask = setTimeout(function() {
						updateElementAndReferences(elem);
						elem._updateCounter --;
						if (elem._updateCounter > 0) {
							elem._updateTask = setTimeout(function() {
								updateElementAndReferences(elem);
								elem._updateCounter --;
							}, 0);
						} else {
							elem._updateTask = false;
						}
					}, 250);
				}
			}
		}
	},
	deleteRow : function(targets, row) {
		for (var i = 0; i < targets.length; i ++) {
			var elem = getForeUIElementById(targets[i]);
			if (elem) {
				if (elem._updateTask) {
					clearTimeout(elem._updateTask);
					elem._updateCounter = 0;
				}
				var params = elem._params;
				var rowCount = elem.getRowCount();
				if (row >= 1 && row <= rowCount) {
					$("[ref=\"" + elem._id + "\"]").add(elem._obj).children(".fg").each(function() {
						var _fg = $(this);
						if (params.rhdr) {
							var rowHeader = _fg.children(".rowHeaderView").children(".rowHeader").children("tbody");
							rowHeader.children().eq(row - 1).remove();
						}
						var contentBody = _fg.children(".contentView").children(".content").children("tbody");
						contentBody.children().eq(row - 1).remove();
					});
				}
				elem._updateCounter = 1;
				elem._updateTask = setTimeout(function() {
					updateElementAndReferences(elem);
					elem._updateCounter --;
					elem._updateTask = false;
				}, 250);
			}
		}
	}
};