var Tabs = function(id, params) {
	this._id = id;
	this._params = params;
	this._obj = $("#" + id);
	this.init();
};

Tabs.prototype = {
	init : function() {
		this._bg = this._obj.children(".bg")
			.css("top", this._params.bd_y + "px");
		this._bd = this._obj.children(".bd")
			.css("top", this._params.bd_y) + "px";
		this._fg = this._obj.children(".fg");
		this.setupBehavior(this._obj);
		initReferences(this);
		var me = this;
		setTimeout(function() {
			setState(me, me._params.state, true);
		}, 50);
	},
	updateUI : function(obj) {
		var params = this._params;
		var state = params.state;
		var _bg = obj.children(".bg");
		var _bd = obj.children(".bd");
		var _fg = obj.children(".fg");
		var width = obj.width();
		var titleHeight = params.title_h;
		var intersect = params.bd_its;
		var height = intersect == "outer" ? obj.height() : (intersect == "inner" ? obj.height() - titleHeight + 1 : obj.height() - titleHeight / 2);
		var swingAmplitude = params["bdsw_amp_" + state];
		
		// background filling
		var viewRc = params.bounds;
		var viewCss = {
			"left" : viewRc.x + "px",
			"top" : viewRc.y + "px",
			"width" : width + "px",
			"height" : height + "px",
			"border-width" : "0px"
		};
		_bg.css(viewCss);
		_bd.css(viewCss);
		generateBackgroundAndBorders(state, "", _bg, _bd, params);
		
		// tab
		var tabAlign = params["tab_a_" + state];
		var tabPaddingPx = params.tab_p + "px";
		var tabCount = params.tab_count;
		var selIndex = params.sel_index;
		var titles = params.tabs;
		var icons = params.icons;
		var gap = params.gap;
		
		var tabsTotalWidth = 0;
		var tabs = _fg.children(".tabs");
		tabs.css({
			"position" : "absolute",
			"width" : "100%"
		});
		if (params.tab_mode == 1) {
			tabs.css("top", (obj.height() - titleHeight - swingAmplitude) + "px");
		}
		
		var tabSpans = tabs.children("span");
		tabSpans.each(function(index) {
			var tab = $(this).empty();
			var tabMargin = params[(index == selIndex ? "stab_m_" : "tab_m_") + state];
			var tabHeight = titleHeight - tabMargin + swingAmplitude;
			
			var label = createLabelObject(icons[index], 0, gap, titles[index]);
			applyFontParameters(state, index == selIndex ? "stab" : "tab", label, params);
			label.css({
				"position" : "absolute",
				"height" : tabHeight + "px",
				"line-height" : tabHeight + "px",
				"display" : "inline-block",
				"padding-left" : tabPaddingPx,
				"padding-right" : tabPaddingPx,
				"vertical-align" : (params.tab_mode == 0 ? "bottom" : "top")
			}).appendTo(tab);
			
			var tabWidthPx = label.innerWidth() + "px";
			var tabHeightPx = tabHeight + "px";
			var tabTop = params.tab_mode == 1 ? 0 : Math.max(0, tabMargin);
			tab.css({
				"position" : "absolute",
				"top" : tabTop + "px",
				"width" : tabWidthPx,
				"height" : tabHeightPx,
				"display" : "inline-block",
				"border-width" : "0px",
				"cursor": (state == 64 || state == 96) ? "default" : "pointer"
			});
			
			var targetId = index == 0 ? "ftab" : (index == tabCount - 1 ? "ltab" : "tab");
			generateBackgroundAndBorders(state, targetId, tab, tab, params);
			if (index == selIndex) {
				var selTabBg = params["stabbg_" + state];
				var selTab = (swingAmplitude == 0 && (selTabBg.indexOf("#") == 0 || selTabBg == "transparent" || selTabBg.indexOf("rgba(") == 0)) ? label : tab;
				changeBackground(state, targetId, selTab, params, selTabBg);
				changeBorders(state, "stab", tab, params);
				if (selTab == label) {
					var bdL = parseInt(tab.css("border-left-width"));
					var bdR = parseInt(tab.css("border-right-width"));
					tab.width(tab.width() + bdL + bdR);
				}
			}
			label.appendTo(tab);
			if (index == selIndex) {
				applyBorderRadius(state, targetId, label, params);
				if (params.tab_mode == 1) {
					var actualTabHeight = tab.height();
					label.css({
						"height" : actualTabHeight + "px",
						"line-height" : actualTabHeight + "px"
					});
				} else {
					setTimeout(function() {
						tab.css("overflow", params.bd_its == "middle" ? "hidden" : "visible");
						tab.children(".linearBg").height(label.height());
					}, 10);
				}
			}
			
			tab.children("img").css("position", "absolute");
			
			tabsTotalWidth += tab.outerWidth();
		});

		// offset tabs according alignment
		var tabOffset;
		if (tabAlign == "left") {
			tabOffset = 0;
		} else if (tabAlign == "center") {
			tabOffset = (width - tabsTotalWidth) / 2;
		} else {
			tabOffset = width - tabsTotalWidth;
		}
		tabSpans.each(function(index) {
			var tab = $(this);
			tab.css("left", tabOffset + "px");
			tabOffset += tab.outerWidth();
		});
		
		// embedded elements
		obj.children(".fg").children(".views").children().each(function(index) {
			if (index == params.sel_index) {
				$(this).css("visibility", "inherit");
			} else {
				$(this).css("visibility", "hidden");
			}
		});
		
		// shadow
		applyShadow(obj, _bd, params.shadow);
	},
	setupBehavior : function(obj) {
		var me = this;
		var tabs = obj.children(".fg").children(".tabs").children("span");
		tabs.click(function() {
			if ((me._params.state & 64) == 0) {
				me.setSelectedIndex(tabs.index(this) + 1);
			}
		});
	},
	setSelectedIndex : function(index) {
		if (this._params.sel_index != index - 1) {
			this._params.sel_index = index - 1;
			updateElementAndReferences(this);
			$("#" + this._id).trigger("Selection_Changed");
		}
	},
	getSelectedIndex : function() {
		return this._params.sel_index + 1;
	}
};